#!/usr/bin/perl -w

use strict;
use warnings;

my $sequenceFileDir  = "010_sequenceFileDir";
my $partitionFileDir = "010_partitionFileDir";
my $jobscript        = "010_jobscript.sh";

my $outDir   = "010_outDir";
my $errorDir = "010_errorDir";

my $resultDir = "020_resDir";  ## Used in 020_autoResultMover.pl


system("mkdir $outDir") unless -d $outDir;
system("mkdir $errorDir") unless -d $errorDir;

opendir(DIR, $sequenceFileDir) or die "Cannot open $sequenceFileDir\n";
my @dirs = readdir(DIR);
close DIR;


#################################################
########### Check the presence of outputfile
print "#### Finished/Running ID checking.\n";
my @selectedInfiles = ( );
my $count = 0;
foreach my $inFile (@dirs) {
  next if $inFile =~ /^\./;
  next if $inFile =~ /\.reduced/;

  my $jobid = $inFile;
  $jobid =~ s/\..*$//;
  my $protID = $jobid;
  $jobid =~ s/(EN)[A-Z]+0+/$1/g;

  my $raxmlBipartitions = "RAxML_bipartitions.".$inFile;
  if(-e "$resultDir/$raxmlBipartitions"){
    print "    $inFile\tFinished.\n";
    next;
  }

  my $raxmlBootstrap = "RAxML_bootstrap.".$inFile;
  if(-e "$raxmlBootstrap"){
    print "    $inFile\tStill running.\n";
    next;
  }

  print "  $count.$inFile\tAdding to the list.\n";
  $count++;

  if($count == 11){
    print "10 jobs listed. Stopped.\n";
    last;
  }

  push(@selectedInfiles, $inFile);
}


#################################################
########### Making job script
print "#### Making the job script.\n\n";

my ($sgeTemp) = &variablesSR;
open(OUT, ">$jobscript");

### Job enviroments
my $totalNum = @selectedInfiles;
$sgeTemp =~ s/TOTALNUM/$totalNum/;
$sgeTemp =~ s/OUTDIR/$outDir/;
$sgeTemp =~ s/ERRORDIR/$errorDir/;
print OUT "$sgeTemp";

foreach my $inFile (@selectedInfiles) {
  ### Picking up the outgroup from each sequence file.
  open(IN, "$sequenceFileDir/$inFile");
  my @inFileCont = <IN>;
  close IN;
  my ($ref_fasHash,$ref_nameArray, $seqLength) = &phyHashReader(\@inFileCont);
  my $outgroup = $ref_nameArray->[0];

  ### Replacing the variables
  my $raxmlLine = './raxmlHPC-PTHREADS-SSE3 -f a -x 12345 -p 12345 -# 100 -m GTRGAMMA -s SEQUENCEFILEDIR/INFILE_TXT -q PARTITIONFILEDIR/INFILE_TXT -o OUTGROUP -n INFILE_TXT -T 2';
  $raxmlLine =~ s/INFILE_TXT/$inFile/g;
  $raxmlLine =~ s/SEQUENCEFILEDIR/$sequenceFileDir/;
  $raxmlLine =~ s/PARTITIONFILEDIR/$partitionFileDir/;
  $raxmlLine =~ s/OUTGROUP/$outgroup/;
  print OUT "$raxmlLine\n";
}


#################################################
########### qsub job script
print "#### qsub job script.\n\n";
system("qsub $jobscript");

close OUT;
exit;

#
#  open(OUT, ">$sgeBatchFileDir/$inFile");
#  print OUT "$sgeTemp\n";
#  close OUT;

exit;

#############################################

sub variablesSR {

  my $sgeTempBlock = '#$ -S /bin/bash 
#$ -t 1-TOTALNUM
#$ -tc 50
#$ -N raxml 
# 
#$ -q shortP 
#$ -M xxx.xxx@oist.jp
#$ -m abe 
#$ -j yes 
#$ -cwd 
#$ -o ./OUTDIR
#$ -e ./ERRORDIR

# #$ -pe openmpi 2
#$ -pe smp 2
#$ -l h_vmem=1g # Maximum memory to be expected
#$ -l virtual_free=1g # Avairable mamory before start job.
#
'; 
  return($sgeTempBlock);
}


sub phyHashReader {
  my $name = "";
  my $seq = "";
  my @nameArray = ( );
  my %phyHash = ( );
  my ($inFileContSR) = @_;

  foreach my $inFileLine (@$inFileContSR) {
    chomp($inFileLine);
    if($inFileLine !~ /^\d/){
      ($name,$seq) = split(/ +/,$inFileLine);
      push(@nameArray,$name);
      $phyHash{$name} = $seq;
    }
  }
  return (\%phyHash,\@nameArray);
}
